<?php
/*
*  Arguments exporter (argexp)
*
*	Used for incoming arguments processing.
*	Accepting: $_GET, $_POST, $_COOKIE, $_SERVER, $_ENV, $GLOBALS.
*
*
*Interfaces:
*
*  mixed get(array $args,string $order);
*	Returns matched value. Check 'exists' property for result.
*
*  string get_type(mixed $value);
*	Returns string of 'type'.
*
*  void to_var(array $args,string $order);
*	Register var into internal array 'var'. Check 'exists' property for result.
*
*  void to_global(array $args,string $order);
*	Register var into $GLOBALS. Check 'exists' property for result.
*
*  void to_var_by_list(array $list,string $order');
*	Register list of vars into internal array 'var'.
*
*  void to_global_by_list(array $list,string $order);
*	Register list of vars into $GLOBALS.
*
*  void to_var_all(string $order);
*	Register all founded vars into internal array 'var'.
*
*  void to_global_all(string $order);
*	Register all founded vars into $GLOBALS.
*
*Properties:
*
*  array var;
*	Internal var's storage.
*
*  boolean exists;
*	Result flag.
*
*  boolean globals_on;
*	Get values from $GLOBALS.
*
*  boolean strip_slashes;
*	Do stripslashes() for strings.
*
*  boolean html_ok;
*	Don't htmlspecialchars() for strings.
*
*Arguments:
*
*  array $arg;
*	[0] - name
*	[1] - noexists action (variable is not exists or type mismatch):
*	    N - no action
*	    U - unset existed var
*	    D - set default value
*	Default is 'D'.
*	[2] - default value
*	Default is ''.
*	[3] - expected type:
*	    S - string
*	    I - integer
*	    D - double
*	    AS - string's array
*	    AI - integer's array
*	    AD - double's array
*	Default is 'S'.
*	If variables's type not compatibled then 'noexists' used.
*
*  string $order;
*	Equvalent of 'variables_order' from php.ini.: Get,Post,Server,Env,Cookie
*	Default is 'GP'.
*
*  array $list;
*	List of $args.
*
*  mixed $value;
*	Scalar or simple array.
*
* Note: 'noexists', 'type' and $order is not case sensived.
*
*
*Examples:
*
*  $args->to_var(array('id','D','0','I'),'G'); // match 'id' in GET only
*  echo("id=".$args->var['id']);
*
*  $args->to_global(array('id','D','0','I')); // match 'id' in GET and POST
*  echo("id=".$id);
*
*  $args->to_var_by_list(array(array('id','D','0','I'), array('sort','','asc')),'G'); // match 'id' and 'sort' in GET
*  echo("id=".$args->var['id']);
*  echo("sort=".$args->var['id']);
*
*
*Author: RXl (rxl@mail.ru)
*
*Changing list:
*
*  2005-03-21 by RXl (rxl@mail.ru)
*	It's wrotted. v0.1
*
*/

class argexp
{

    var $var;
    var $exists;
    var $globals_on;
    var $strip_slashes;
    var $html_ok;

    function argexp()
    {
	$this->var = array();
	$this->globals_on = ini_get('register_globals'); // get from $GLOBALS
	$this->strip_slashes = ini_get('magic_quotes_gpc'); // do stripslashes()
	$this->html_ok = 0; // no HTML tags by default
    }

    function get_type($value)
    {
	if(is_array($value))
	{
	    $at = '';
	    foreach($value as $v)
	    {
		if(is_array($v)) die('DEBUG: $args->get_type(): recursive arrays<BR>');
		$et = $this->get_type($v);
		if($at == '') $at = $et;
		else if($at != $et)
		{
		    if(($at == 'D' || $at == 'I') && ($et == 'D' || $et == 'I')) $at = 'D';
		    return 'AS';
		}
	    }
	    return 'A'.$at;
	}
	else
	{
	    if(preg_match('/^\s*([-+]?\d+)\s*$/',$value)) return 'I';
	    else if(preg_match('/^\s*([-+]?(?:\d+(?:\.\d*)?|\d*\.\d+)(?:E[-+]?\d+)?)\s*$/i',$value)) return 'D';
	    else return 'S';
	}
    }

    function get($arg,$order='GP')
    {
    	if(!is_array($arg)) die('DEBUG: $args->get(): wrong arguments<BR>');
	if(!isset($arg[1])) $arg[1]='D';
	if(!isset($arg[2])) $arg[2]='';
	if(!isset($arg[3])) $arg[3]='S';
	list($name,$noexists,$default,$type) = $arg;
	$type = strtoupper($type);
	$noexists = strtoupper($noexists);
	$order = strtoupper($order);
	if($this->globals_on == '1')
	{
	    if(isset($GLOBALS[$name])) $r= $GLOBALS[$name];
	}
	else
	{
	    for($i=0;$i<strlen($order);$i++)
	    {
		switch(substr($order,$i,1))
		{
		case 'P':
		    if(isset($_POST[$name])) $r = $_POST[$name];
		    break;
		case 'G':
		    if(isset($_GET[$name])) $r = $_GET[$name];
		    break;
		case 'S':
		    if(isset($_SERVER[$name])) $r = $_SERVER[$name];
		    break;
		case 'C':
		    if(isset($_COOKIE[$name])) $r = $_COOKIE[$name];
		    break;
		case 'E':
		    if(isset($_ENV[$name])) $r = $_ENV[$name];
		    break;
		}
	    }
	}
	$f=0;
	if(!isset($r)) $f=1;
	else
	{
	    switch($type) // type check
	    {
	    case 'I':
		if($this->get_type($r) != 'I') $f=1;
		break;
	    case 'D':
		if(($tt = $this->get_type($r)) != 'D' && $tt != 'I') $f=1;
		break;
	    case 'AS':
		if(substr($this->get_type($r),0,1) != 'A') $f=1; // string or mixed array
		else
		{
		    foreach($r as $k=>$v)
		    {
			if($this->strip_slashes) $r[$k] = stripslashes($v);
			if(!$this->html_ok) $r[$k] = htmlspecialchars($v,ENT_QUOTES);
		    }
		}
		break;
	    case 'AI':
		if($this->get_type($r) != 'AI') $f=1;
		break;
	    case 'AD':
		if(($tt = $this->get_type($r)) != 'AD' && $tt != 'AI') $f=1;
		break;
	    default: // string
		if($this->strip_slashes) $r = stripslashes($r);
		if(!$this->html_ok) $r = htmlspecialchars($r,ENT_QUOTES);
	    }
	}
	if($f)
	{
	    switch($noexists) // not exists or type mismatch
	    {
	    case 'N':
	    case 'U':
		$this->exists=0;
		return;
	    case 'D':
		if(substr($type,0,1) == 'A')
		{
		    if(is_array($default)) $r = $default;
		    else $r = array();
		}
		else $r = $default;
		break;
	    }
	}
	$this->exists=1;
	return $r;
    }

    function to_global($arg,$order='GP')
    {
	$r = $this->get($arg,$order);
	if($this->exists) $GLOBALS[$arg[0]] = $r;
	else if(strtoupper($arg[1]) == 'U') unset($GLOBALS[$arg[0]]);
    }

    function to_var($arg,$order='GP')
    {
	$r = $this->get($arg,$order='GP');
	if($this->exists) $this->var[$arg[0]] = $r;
	else if(strtoupper($arg[1]) == 'U') unset($this->var[$arg[0]]);
    }

    function to_global_by_list($list,$order='GP')
    {
	if(!is_array($list)) die('DEBUG: $args->to_global_by_list(): wrong arguments<BR>');
	$temp = $this->var;
	$this->var = array();
	$this->to_var_by_list($list,$order);
	foreach($this->var as $name=>$val) $GLOBALS[$name] = $val;
	$this->var = $temp;
    }

    function to_var_by_list($list,$order='GP')
    {
	if(!is_array($list)) die('DEBUG: $args->to_var_by_list(): wrong arguments<BR>');
	foreach($list as $arg)
	{
	    $this->to_var($arg,$order);
	}
    }

    function to_global_all($order="EGPCS")
    {
	$temp = $this->var;
	$this->var = array();
	$this->to_var_all($order);
	foreach($this->var as $name=>$val) $GLOBALS[$name] = $val;
	$this->var = $temp;
    }

    function to_var_all($order="EGPCS")
    {
	for($i=0;$i<strlen($order);$i++)
	{
	    unset($r);
	    switch(strtoupper(substr($order,$i,1)))
	    {
	    case 'P':
		if(!isset($_POST)) break;
		foreach($_POST as $name=>$val)
		{
		    if(isset($_POST[$name])) $r = $val;
		}
	        break;
	    case 'G':
		if(!isset($_GET)) break;
		foreach($_GET as $name=>$val)
		{
		    if(isset($_GET[$name])) $r = $val;
		}
	        break;
	    case 'S':
		if(!isset($_SERVER)) break;
		foreach($_SERVER as $name=>$val)
		{
		    if(isset($_SERVER[$name])) $r = $val;
		}
	        break;
	    case 'C':
		if(!isset($_COOKIE)) break;
		foreach($_COOKIE as $name=>$val)
		{
		    if(isset($_COOKIE[$name])) $r = $val;
		}
	        break;
	    case 'E':
		if(!isset($_ENV)) break;
		foreach($_ENV as $name=>$val)
		{
		    if(isset($_ENV[$name])) $r = $val;
		}
	        break;
	    }
	    if(isset($r))
	    {
		if($this->strip_slashes) $r = stripslashes($r);
		if(!$this->html_ok) $r = htmlspecialchars($r,ENT_QUOTES);
		$this->var[$name] = $r;
	    }
	}
    }
}

$argx = new argexp;

?>